@@ -508,7 +508,7 @@ eapttls_chap(EAPClientPluginDataRef plugin)
     void *		data;
     int			data_length;
     int			data_length_r;
-    char		key_data[17];
+    uint8_t		key_data[17];
     size_t		length;
     void *		offset;
     bool		ret = TRUE;
@@ -611,11 +611,11 @@ eapttls_mschap(EAPClientPluginDataRef plugin)
     void *		data;
     int			data_length;
     int			data_length_r;
-    char		key_data[MSCHAP_NT_CHALLENGE_SIZE + MSCHAP_IDENT_SIZE];
+    uint8_t		key_data[MSCHAP_NT_CHALLENGE_SIZE + MSCHAP_IDENT_SIZE];
     size_t		length;
     void *		offset;
     bool		ret = TRUE;
-    char		response[MSCHAP_NT_RESPONSE_SIZE];
+    uint8_t		response[MSCHAP_NT_RESPONSE_SIZE];
     OSStatus		status;
     int			user_length_r;
 
@@ -1449,6 +1449,7 @@ static CFDictionaryRef
 eapttls_publish_props(EAPClientPluginDataRef plugin)
 {
     CFArrayRef			cert_list;
+    SSLCipherSuite		cipher = SSL_NULL_WITH_NULL_NULL;
     EAPTTLSPluginDataRef	context = (EAPTTLSPluginDataRef)plugin->private;
     CFMutableDictionaryRef	dict;
 
@@ -1469,15 +1470,25 @@ eapttls_publish_props(EAPClientPluginDataRef plugin)
 			 ? kCFBooleanTrue
 			 : kCFBooleanFalse);
     my_CFRelease(&cert_list);
+    (void)SSLGetNegotiatedCipher(context->ssl_context, &cipher);
+    if (cipher != SSL_NULL_WITH_NULL_NULL) {
+	CFNumberRef	c;
+
+	c = CFNumberCreate(NULL, kCFNumberIntType, &cipher);
+	CFDictionarySetValue(dict, kEAPClientPropTLSNegotiatedCipher, c);
+	CFRelease(c);
+    }
     if (context->last_client_status == kEAPClientStatusUserInputRequired
 	&& context->trust_proceed == FALSE) {
 	CFNumberRef	num;
 	num = CFNumberCreate(NULL, kCFNumberSInt32Type,
 			     &context->trust_status);
 	CFDictionarySetValue(dict, kEAPClientPropTLSTrustClientStatus, num);
+	CFRelease(num);
 	num = CFNumberCreate(NULL, kCFNumberSInt32Type,
 			     &context->trust_proceed_id);
 	CFDictionarySetValue(dict, kEAPClientPropTLSUserTrustProceed, num);
+	CFRelease(num);
     }
     return (dict);
 }
