@@ -82,6 +82,7 @@ PEAPPacketFlagsVersion(uint8_t flags)
 static __inline__ void
 PEAPPacketFlagsSetVersion(EAPTLSPacketRef eap_tls, uint8_t version)
 {
+    eap_tls->flags &= ~kPEAPPacketFlagsVersionMask;
     eap_tls->flags |= PEAPPacketFlagsVersion(version);
     return;
 }
@@ -1540,6 +1541,7 @@ static CFDictionaryRef
 peap_publish_props(EAPClientPluginDataRef plugin)
 {
     CFArrayRef			cert_list;
+    SSLCipherSuite		cipher = SSL_NULL_WITH_NULL_NULL;
     PEAPPluginDataRef		context = (PEAPPluginDataRef)plugin->private;
     CFMutableDictionaryRef	dict;
 
@@ -1565,6 +1567,14 @@ peap_publish_props(EAPClientPluginDataRef plugin)
 			 context->session_was_resumed 
 			 ? kCFBooleanTrue
 			 : kCFBooleanFalse);
+    (void)SSLGetNegotiatedCipher(context->ssl_context, &cipher);
+    if (cipher != SSL_NULL_WITH_NULL_NULL) {
+	CFNumberRef	c;
+	
+	c = CFNumberCreate(NULL, kCFNumberIntType, &cipher);
+	CFDictionarySetValue(dict, kEAPClientPropTLSNegotiatedCipher, c);
+	CFRelease(c);
+    }
     my_CFRelease(&cert_list);
     if (context->eap.module != NULL) {
 	dictInsertEAPTypeInfo(dict, context->eap.last_type,
@@ -1576,9 +1586,11 @@ peap_publish_props(EAPClientPluginDataRef plugin)
 	num = CFNumberCreate(NULL, kCFNumberSInt32Type,
 			     &context->trust_status);
 	CFDictionarySetValue(dict, kEAPClientPropTLSTrustClientStatus, num);
+	CFRelease(num);
 	num = CFNumberCreate(NULL, kCFNumberSInt32Type,
 			     &context->trust_proceed_id);
 	CFDictionarySetValue(dict, kEAPClientPropTLSUserTrustProceed, num);
+	CFRelease(num);
     }
     return (dict);
 }
